@@ -17,6 +17,7 @@
 #
 from __future__ import absolute_import, division, print_function
 
+
 __metaclass__ = type
 
 
@@ -24,7 +25,7 @@ DOCUMENTATION = """
 module: nxos_vrf_interface
 extends_documentation_fragment:
 - cisco.nxos.nxos
-short_description: Manages interface specific VRF configuration.
+short_description: (deprecated, removed after 2028-06-01) Manages interface specific VRF configuration.
 description:
 - Manages interface specific VRF configuration.
 version_added: 1.0.0
@@ -83,19 +84,15 @@ commands:
 """
 import re
 
-from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
-    load_config,
-    run_commands,
-)
+from ansible.module_utils.basic import AnsibleModule
+
 from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
     get_capabilities,
-    nxos_argument_spec,
-    normalize_interface,
-)
-from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
     get_interface_type,
+    load_config,
+    normalize_interface,
+    run_commands,
 )
-from ansible.module_utils.basic import AnsibleModule
 
 
 def execute_show_command(command, module):
@@ -184,16 +181,10 @@ def main():
     argument_spec = dict(
         vrf=dict(required=True),
         interface=dict(type="str", required=True),
-        state=dict(
-            default="present", choices=["present", "absent"], required=False
-        ),
+        state=dict(default="present", choices=["present", "absent"], required=False),
     )
 
-    argument_spec.update(nxos_argument_spec)
-
-    module = AnsibleModule(
-        argument_spec=argument_spec, supports_check_mode=True
-    )
+    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
 
     warnings = list()
     results = {"changed": False, "commands": [], "warnings": warnings}
@@ -207,10 +198,7 @@ def main():
 
     current_vrfs = get_vrf_list(module)
     if vrf not in current_vrfs:
-        warnings.append(
-            "The VRF is not present/active on the device. "
-            "Use nxos_vrf to fix this."
-        )
+        warnings.append("The VRF is not present/active on the device. Use nxos_vrf to fix this.")
 
     intf_type = get_interface_type(interface)
     if intf_type != "ethernet" and network_api == "cliconf":
@@ -218,7 +206,7 @@ def main():
             module.fail_json(
                 msg="interface does not exist on switch. Verify "
                 "switch platform or create it first with "
-                "nxos_interface if it's a logical interface"
+                "nxos_interfaces if it's a logical interface",
             )
 
     mode = get_interface_mode(interface, intf_type, module)
@@ -226,7 +214,7 @@ def main():
         module.fail_json(
             msg="Ensure interface is a Layer 3 port before "
             "configuring a VRF on an interface. You can "
-            "use nxos_interface"
+            "use nxos_interfaces",
         )
 
     current_vrf = get_interface_info(interface, module)
