@@ -2,8 +2,8 @@
 
 #
 # Dell OpenManage Ansible Modules
-# Version 7.0.0
-# Copyright (C) 2019-2020 Dell Inc. or its subsidiaries. All Rights Reserved.
+# Version 10.0.1
+# Copyright (C) 2019-2023 Dell Inc. or its subsidiaries. All Rights Reserved.
 
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 #
@@ -31,7 +31,8 @@ def ome_connection_mock_for_template(mocker, ome_response_mock):
     connection_class_mock = mocker.patch(MODULE_PATH + 'RestOME')
     ome_connection_mock_obj = connection_class_mock.return_value.__enter__.return_value
     ome_connection_mock_obj.invoke_request.return_value = ome_response_mock
-    ome_connection_mock_obj.get_all_report_details.return_value = {"report_list": []}
+    ome_connection_mock_obj.get_all_report_details.return_value = {
+        "report_list": []}
     return ome_connection_mock_obj
 
 
@@ -51,8 +52,10 @@ class TestOmeTemplate(FakeAnsibleModule):
         ome_connection_mock_for_template.get_all_report_details.return_value = {
             "report_list": [{"Id": Constants.device_id1,
                              "DeviceServiceTag": Constants.service_tag1}]}
-        f_module = self.get_module_mock({'device_id': [], 'device_service_tag': [Constants.service_tag1]})
-        data = self.module.get_device_ids(f_module, ome_connection_mock_for_template)
+        f_module = self.get_module_mock(
+            {'device_id': [], 'device_service_tag': [Constants.service_tag1]})
+        data = self.module.get_device_ids(
+            f_module, ome_connection_mock_for_template)
         assert data == [Constants.device_id1]
 
     def test_get_device_ids_failure_case01(self, ome_connection_mock_for_template, ome_response_mock, ome_default_args):
@@ -60,7 +63,8 @@ class TestOmeTemplate(FakeAnsibleModule):
         ome_response_mock.success = False
         f_module = self.get_module_mock(params={'device_id': ["#@!1"]})
         with pytest.raises(Exception) as exc:
-            self.module.get_device_ids(f_module, ome_connection_mock_for_template)
+            self.module.get_device_ids(
+                f_module, ome_connection_mock_for_template)
         assert exc.value.args[0] == "Unable to complete the operation because the entered target device id(s) " \
                                     "'{0}' are invalid.".format("#@!1")
 
@@ -205,9 +209,11 @@ class TestOmeTemplate(FakeAnsibleModule):
                             {"Id": Constants.device_id2,
                              "DeviceServiceTag": "tag2"}
                             ]}
-        f_module = self.get_module_mock(params={'device_id': [Constants.device_id2], 'device_service_tag': ["abcd"]})
+        f_module = self.get_module_mock(
+            params={'device_id': [Constants.device_id2], 'device_service_tag': ["abcd"]})
         with pytest.raises(Exception) as exc:
-            self.module.get_device_ids(f_module, ome_connection_mock_for_template)
+            self.module.get_device_ids(
+                f_module, ome_connection_mock_for_template)
         assert exc.value.args[0] == "Unable to complete the operation because the entered target service tag(s) " \
                                     "'{0}' are invalid.".format('abcd')
 
@@ -217,9 +223,11 @@ class TestOmeTemplate(FakeAnsibleModule):
             "report_list": [{"Id": Constants.device_id1,
                              "DeviceServiceTag": Constants.service_tag1}
                             ], "resp_obj": ome_response_mock}
-        f_module = self.get_module_mock(params={'device_service_tag': [Constants.service_tag1], 'device_id': []})
+        f_module = self.get_module_mock(
+            params={'device_service_tag': [Constants.service_tag1], 'device_id': []})
         with pytest.raises(Exception) as exc:
-            device_ids = self.module.get_device_ids(f_module, ome_connection_mock_for_template)
+            device_ids = self.module.get_device_ids(
+                f_module, ome_connection_mock_for_template)
             assert exc.value.args[0] == "Failed to fetch the device ids."
 
     def test_get_view_id_success_case(self, ome_connection_mock_for_template, ome_response_mock):
@@ -237,7 +245,8 @@ class TestOmeTemplate(FakeAnsibleModule):
                                         "SourceDeviceId": 2224}])
     def test_get_create_payload(self, param, ome_response_mock, ome_connection_mock_for_template):
         f_module = self.get_module_mock(params=param)
-        data = self.module.get_create_payload(f_module, ome_connection_mock_for_template, 2224, 4)
+        data = self.module.get_create_payload(
+            f_module, ome_connection_mock_for_template, 2224, 4)
         assert data['Fqdds'] == "All"
 
     def test_get_template_by_id_success_case(self, ome_response_mock):
@@ -249,7 +258,8 @@ class TestOmeTemplate(FakeAnsibleModule):
         assert data
 
     def test_get_template_by_name_success_case(self, ome_response_mock, ome_connection_mock_for_template):
-        ome_response_mock.json_data = {'value': [{"Name": "test Sample Template import1", "Id": 24}]}
+        ome_response_mock.json_data = {
+            'value': [{"Name": "test Sample Template import1", "Id": 24}]}
         ome_response_mock.status_code = 200
         ome_response_mock.success = True
         f_module = self.get_module_mock()
@@ -259,20 +269,24 @@ class TestOmeTemplate(FakeAnsibleModule):
         assert data["Id"] == 24
 
     def test_get_group_devices_all(self, ome_response_mock, ome_connection_mock_for_template):
-        ome_response_mock.json_data = {'value': [{"Name": "Device1", "Id": 24}]}
+        ome_response_mock.json_data = {
+            'value': [{"Name": "Device1", "Id": 24}]}
         ome_response_mock.status_code = 200
         ome_response_mock.success = True
         f_module = self.get_module_mock()
-        data = self.module.get_group_devices_all(ome_connection_mock_for_template, "uri")
+        data = self.module.get_group_devices_all(
+            ome_connection_mock_for_template, "uri")
         assert data == [{"Name": "Device1", "Id": 24}]
 
     def _test_get_template_by_name_fail_case(self, ome_response_mock):
-        ome_response_mock.json_data = {'value': [{"Name": "template by name for template name", "Id": 12}]}
+        ome_response_mock.json_data = {
+            'value': [{"Name": "template by name for template name", "Id": 12}]}
         ome_response_mock.status_code = 500
         ome_response_mock.success = False
         f_module = self.get_module_mock()
         with pytest.raises(Exception) as exc:
-            self.module.get_template_by_name("template by name for template name", f_module, ome_response_mock)
+            self.module.get_template_by_name(
+                "template by name for template name", f_module, ome_response_mock)
         assert exc.value.args[0] == "Unable to complete the operation because the" \
                                     " requested template with name {0} is not present." \
             .format("template by name for template name")
@@ -305,7 +319,8 @@ class TestOmeTemplate(FakeAnsibleModule):
                      return_value=["Deployment"])
         mocker.patch(MODULE_PATH + 'get_create_payload',
                      return_value=params["mid"])
-        data = self.module._get_resource_parameters(f_module, ome_connection_mock_for_template)
+        data = self.module._get_resource_parameters(
+            f_module, ome_connection_mock_for_template)
         assert data == params["out"]
 
     modify_payload = {"command": "modify", "device_id": [25007], "template_id": 1234,
@@ -334,68 +349,90 @@ class TestOmeTemplate(FakeAnsibleModule):
                      return_value={})
         mocker.patch(MODULE_PATH + 'get_modify_payload',
                      return_value={})
-        mocker.patch(MODULE_PATH + 'get_template_details', return_value={"Id": 1234, "Name": "templ1"})
-        data = self.module._get_resource_parameters(f_module, ome_connection_mock_for_template)
+        mocker.patch(MODULE_PATH + 'get_template_details',
+                     return_value={"Id": 1234, "Name": "templ1"})
+        data = self.module._get_resource_parameters(
+            f_module, ome_connection_mock_for_template)
         assert data == ('TemplateService/Templates(1234)', {}, 'PUT')
 
     def test__get_resource_parameters_delete_success_case(self, mocker, ome_response_mock,
                                                           ome_connection_mock_for_template):
-        f_module = self.get_module_mock({"command": "delete", "template_id": 1234})
-        mocker.patch(MODULE_PATH + 'get_template_details', return_value={"Id": 1234, "Name": "templ1"})
-        data = self.module._get_resource_parameters(f_module, ome_connection_mock_for_template)
+        f_module = self.get_module_mock(
+            {"command": "delete", "template_id": 1234})
+        mocker.patch(MODULE_PATH + 'get_template_details',
+                     return_value={"Id": 1234, "Name": "templ1"})
+        data = self.module._get_resource_parameters(
+            f_module, ome_connection_mock_for_template)
         assert data == ('TemplateService/Templates(1234)', {}, 'DELETE')
 
     def test__get_resource_parameters_export_success_case(self, mocker, ome_response_mock,
                                                           ome_connection_mock_for_template):
-        f_module = self.get_module_mock({"command": "export", "template_id": 1234})
-        mocker.patch(MODULE_PATH + 'get_template_details', return_value={"Id": 1234, "Name": "templ1"})
-        data = self.module._get_resource_parameters(f_module, ome_connection_mock_for_template)
-        assert data == ('TemplateService/Actions/TemplateService.Export', {'TemplateId': 1234}, 'POST')
+        f_module = self.get_module_mock(
+            {"command": "export", "template_id": 1234})
+        mocker.patch(MODULE_PATH + 'get_template_details',
+                     return_value={"Id": 1234, "Name": "templ1"})
+        data = self.module._get_resource_parameters(
+            f_module, ome_connection_mock_for_template)
+        assert data == (
+            'TemplateService/Actions/TemplateService.Export', {'TemplateId': 1234}, 'POST')
 
     def test__get_resource_parameters_deploy_success_case(self, mocker, ome_response_mock,
                                                           ome_connection_mock_for_template):
-        f_module = self.get_module_mock({"command": "deploy", "template_id": 1234})
+        f_module = self.get_module_mock(
+            {"command": "deploy", "template_id": 1234})
         mocker.patch(MODULE_PATH + 'get_device_ids',
                      return_value=[Constants.device_id1])
         mocker.patch(MODULE_PATH + 'get_deploy_payload',
                      return_value={"deploy_payload": "value"})
-        mocker.patch(MODULE_PATH + 'get_template_details', return_value={"Id": 1234, "Name": "templ1"})
-        data = self.module._get_resource_parameters(f_module, ome_connection_mock_for_template)
-        assert data == ('TemplateService/Actions/TemplateService.Deploy', {"deploy_payload": "value"}, 'POST')
+        mocker.patch(MODULE_PATH + 'get_template_details',
+                     return_value={"Id": 1234, "Name": "templ1"})
+        data = self.module._get_resource_parameters(
+            f_module, ome_connection_mock_for_template)
+        assert data == ('TemplateService/Actions/TemplateService.Deploy',
+                        {"deploy_payload": "value"}, 'POST')
 
     def test__get_resource_parameters_clone_success_case(self, mocker, ome_response_mock,
                                                          ome_connection_mock_for_template):
-        f_module = self.get_module_mock({"command": "clone", "template_id": 1234, "template_view_type": 2})
+        f_module = self.get_module_mock(
+            {"command": "clone", "template_id": 1234, "template_view_type": 2})
         mocker.patch(MODULE_PATH + 'get_view_id',
                      return_value=2)
         mocker.patch(MODULE_PATH + 'get_clone_payload',
                      return_value={"clone_payload": "value"})
-        mocker.patch(MODULE_PATH + 'get_template_details', return_value={"Id": 1234, "Name": "templ1"})
-        data = self.module._get_resource_parameters(f_module, ome_connection_mock_for_template)
-        assert data == ('TemplateService/Actions/TemplateService.Clone', {"clone_payload": "value"}, 'POST')
+        mocker.patch(MODULE_PATH + 'get_template_details',
+                     return_value={"Id": 1234, "Name": "templ1"})
+        data = self.module._get_resource_parameters(
+            f_module, ome_connection_mock_for_template)
+        assert data == ('TemplateService/Actions/TemplateService.Clone',
+                        {"clone_payload": "value"}, 'POST')
 
     def test__get_resource_parameters_import_success_case(self, mocker, ome_response_mock,
                                                           ome_connection_mock_for_template):
-        f_module = self.get_module_mock({"command": "import", "template_id": 1234, "template_view_type": 2})
+        f_module = self.get_module_mock(
+            {"command": "import", "template_id": 1234, "template_view_type": 2})
         mocker.patch(MODULE_PATH + 'get_view_id',
                      return_value=2)
         mocker.patch(MODULE_PATH + 'get_import_payload',
                      return_value={"import_payload": "value"})
-        data = self.module._get_resource_parameters(f_module, ome_connection_mock_for_template)
-        assert data == ('TemplateService/Actions/TemplateService.Import', {"import_payload": "value"}, 'POST')
+        data = self.module._get_resource_parameters(
+            f_module, ome_connection_mock_for_template)
+        assert data == ('TemplateService/Actions/TemplateService.Import',
+                        {"import_payload": "value"}, 'POST')
 
     @pytest.mark.parametrize("params", [{"inp": {"command": "modify"}, "mid": inter_payload, "out": payload_out}])
     def test__get_resource_parameters_modify_template_none_failure_case(self, mocker, ome_response_mock,
                                                                         ome_connection_mock_for_template, params):
         f_module = self.get_module_mock(params=params["inp"])
         with pytest.raises(Exception) as exc:
-            data = self.module._get_resource_parameters(f_module, ome_connection_mock_for_template)
+            data = self.module._get_resource_parameters(
+                f_module, ome_connection_mock_for_template)
         assert exc.value.args[0] == "Enter a valid template_name or template_id"
 
     @pytest.mark.parametrize("params",
                              [{"success": True, "json_data": {"value": [{"Name": "template_name", "Id": 123}]},
                                "id": 123, "gtype": True},
-                              {"success": True, "json_data": {}, "id": 0, "gtype": False},
+                              {"success": True, "json_data": {},
+                                  "id": 0, "gtype": False},
                               {"success": False, "json_data": {"value": [{"Name": "template_name", "Id": 123}]},
                                "id": 0, "gtype": False},
                               {"success": True, "json_data": {"value": [{"Name": "template_name1", "Id": 123}]},
@@ -404,13 +441,15 @@ class TestOmeTemplate(FakeAnsibleModule):
                                ome_response_mock):
         ome_response_mock.success = params["success"]
         ome_response_mock.json_data = params["json_data"]
-        id = self.module.get_type_id_valid(ome_connection_mock_for_template, params["id"])
+        id = self.module.get_type_id_valid(
+            ome_connection_mock_for_template, params["id"])
         assert id == params["gtype"]
 
     @pytest.mark.parametrize("params",
                              [{"success": True, "json_data": {"value": [{"Description": "Deployment", "Id": 2}]},
                                "view": "Deployment", "gtype": 2},
-                              {"success": True, "json_data": {}, "view": "Compliance", "gtype": 1},
+                              {"success": True, "json_data": {},
+                                  "view": "Compliance", "gtype": 1},
                               {"success": False, "json_data": {"value": [{"Description": "template_name", "Id": 1}]},
                                "view": "Deployment", "gtype": 2},
                               {"success": True, "json_data": {"value": [{"Description": "template_name1", "Id": 2}]},
@@ -419,12 +458,14 @@ class TestOmeTemplate(FakeAnsibleModule):
                          ome_response_mock):
         ome_response_mock.success = params["success"]
         ome_response_mock.json_data = params["json_data"]
-        id = self.module.get_view_id(ome_connection_mock_for_template, params["view"])
+        id = self.module.get_view_id(
+            ome_connection_mock_for_template, params["view"])
         assert id == params["gtype"]
 
     @pytest.mark.parametrize("param",
                              [{"pin": {"NetworkBootIsoModel": {"ShareDetail": {"Password": "share_password"}}}},
-                              {"pin": {"NetworkBootIsoModel": {"ShareDetail": {"Password1": "share_password"}}}},
+                              {"pin": {"NetworkBootIsoModel": {
+                                  "ShareDetail": {"Password1": "share_password"}}}},
                               {"pin": {"NetworkBootIsoModel": {"ShareDetail": [{"Password1": "share_password"}]}}}])
     def test_password_no_log(self, param):
         attributes = param["pin"]
@@ -432,13 +473,15 @@ class TestOmeTemplate(FakeAnsibleModule):
 
     def test__get_resource_parameters_create_failure_case_02(self, mocker, ome_response_mock,
                                                              ome_connection_mock_for_template):
-        f_module = self.get_module_mock({"command": "create", "template_name": "name"})
+        f_module = self.get_module_mock(
+            {"command": "create", "template_name": "name"})
         mocker.patch(MODULE_PATH + 'get_device_ids',
                      return_value=[Constants.device_id1, Constants.device_id2])
         mocker.patch(MODULE_PATH + 'get_template_by_name',
                      return_value=("template", 1234))
         with pytest.raises(Exception) as exc:
-            data = self.module._get_resource_parameters(f_module, ome_connection_mock_for_template)
+            data = self.module._get_resource_parameters(
+                f_module, ome_connection_mock_for_template)
         assert exc.value.args[0] == "Create template requires only one reference device"
 
     def test_main_template_success_case2(self, ome_default_args, mocker, module_mock, ome_connection_mock_for_template,
@@ -453,17 +496,22 @@ class TestOmeTemplate(FakeAnsibleModule):
         ome_response_mock.success = True
         mocker.patch(MODULE_PATH + '_get_resource_parameters',
                      return_value=(TEMPLATE_RESOURCE, "template_payload", "POST"))
+        mocker.patch(MODULE_PATH + 'time.sleep', return_value=None)
         result = self._run_module(ome_default_args)
         assert result['changed'] is True
-        assert result['msg'] == "Successfully created a template with ID {0}".format(ome_response_mock.json_data)
+        assert result['msg'] == "Successfully created a template with ID {0}".format(
+            ome_response_mock.json_data)
 
     def test_get_import_payload_success_case_01(self, ome_connection_mock_for_template):
-        f_module = self.get_module_mock(params={"attributes": {"Name": "template1", "Content": "Content"}})
-        self.module.get_import_payload(f_module, ome_connection_mock_for_template, 2)
+        f_module = self.get_module_mock(
+            params={"attributes": {"Name": "template1", "Content": "Content"}})
+        self.module.get_import_payload(
+            f_module, ome_connection_mock_for_template, 2)
 
     def test_get_deploy_payload_success_case_01(self):
         module_params = {"attributes": {"Name": "template1"}}
-        self.module.get_deploy_payload(module_params, [Constants.device_id1], 1234)
+        self.module.get_deploy_payload(
+            module_params, [Constants.device_id1], 1234)
 
     @pytest.mark.parametrize("param",
                              [{"mparams": {"attributes": {"Name": "template1"}}, "name": "template0",
@@ -473,7 +521,8 @@ class TestOmeTemplate(FakeAnsibleModule):
     def test_get_clone_payload_success_case_01(self, param, ome_connection_mock_for_template):
         f_module = self.get_module_mock(param["mparams"])
         module_params = param["mparams"]
-        payload = self.module.get_clone_payload(f_module, ome_connection_mock_for_template, param['template_id'], 2)
+        payload = self.module.get_clone_payload(
+            f_module, ome_connection_mock_for_template, param['template_id'], 2)
         assert payload == param['clone_payload']
 
     @pytest.mark.parametrize("param",
@@ -511,8 +560,10 @@ class TestOmeTemplate(FakeAnsibleModule):
         ome_response_mock.json_data = {
             "value": [{'Id': 1, "Name": "mygroup3"}, {'Id': 2, "Name": "mygroup2"}, {'Id': 3, "Name": "mygroup"}]}
         ome_response_mock.status_code = 200
-        mocker.patch(MODULE_PATH + 'get_group_devices_all', return_value=[{'Id': 1}, {'Id': 2}, {'Id': 3}])
-        dev_list = self.module.get_group_details(ome_connection_mock_for_template, f_module)
+        mocker.patch(MODULE_PATH + 'get_group_devices_all',
+                     return_value=[{'Id': 1}, {'Id': 2}, {'Id': 3}])
+        dev_list = self.module.get_group_details(
+            ome_connection_mock_for_template, f_module)
         assert dev_list == param["dev_list"]
 
     @pytest.mark.parametrize("param", [
@@ -526,8 +577,10 @@ class TestOmeTemplate(FakeAnsibleModule):
         ome_response_mock.json_data = {
             "value": [{'Id': 1, "Name": "mygroup3"}, {'Id': 2, "Name": "mygroup2"}, {'Id': 3, "Name": "mygroup"}]}
         ome_response_mock.status_code = 200
-        mocker.patch(MODULE_PATH + 'get_group_devices_all', return_value=[{'Id': 1}, {'Id': 2}, {'Id': 3}])
-        dev_list = self.module.get_group_details(ome_connection_mock_for_template, f_module)
+        mocker.patch(MODULE_PATH + 'get_group_devices_all',
+                     return_value=[{'Id': 1}, {'Id': 2}, {'Id': 3}])
+        dev_list = self.module.get_group_details(
+            ome_connection_mock_for_template, f_module)
         assert dev_list == param["dev_list"]
 
     @pytest.mark.parametrize("params", [
@@ -567,36 +620,151 @@ class TestOmeTemplate(FakeAnsibleModule):
                             ome_response_mock):
         ome_response_mock.success = params.get("success", True)
         ome_response_mock.json_data = params["json_data"]
-        mocker.patch(MODULE_PATH + 'get_template_by_name', return_value=params.get('get_template_by_name'))
-        mocker.patch(MODULE_PATH + 'attributes_check', return_value=params.get('attributes_check', 0))
-        f_module = self.get_module_mock(params=params["mparams"], check_mode=params.get('check_mode', False))
+        mocker.patch(MODULE_PATH + 'get_template_by_name',
+                     return_value=params.get('get_template_by_name'))
+        mocker.patch(MODULE_PATH + 'attributes_check',
+                     return_value=params.get('attributes_check', 0))
+        f_module = self.get_module_mock(
+            params=params["mparams"], check_mode=params.get('check_mode', False))
         error_message = params["res"]
         with pytest.raises(Exception) as err:
-            self.module.get_modify_payload(f_module, ome_connection_mock_for_template, params.get('template'))
+            self.module.get_modify_payload(
+                f_module, ome_connection_mock_for_template, params.get('template'))
         assert err.value.args[0] == error_message
 
+    @pytest.mark.parametrize("params", [
+        {"json_data": {"value": [
+            {'Id': 123, 'TargetId': 123, 'ProfileState': 1,
+                'DeviceId': 1234, "Type": 1000},
+            {'Id': 234, 'TargetId': 235, 'ProfileState': 1, 'DeviceId': 1235, "Type": 1000}],
+            "report_list": [{'Id': 1234, 'PublicAddress': "XX.XX.XX.XX",
+                             'DeviceId': 1234, "Type": 1000}]},
+         "job_tracking": (True, "msg", {'LastRunStatus': {"Name": "Running"}}, True),
+            'message': "Template operation is in progress. Task excited after 'job_wait_timeout'.",
+            'mparams': {"command": "deploy", "template_id": 123, "device_id": 1234}
+         },
+        {"json_data": {"value": [
+            {'Id': 123, 'TargetId': 123, 'ProfileState': 1,
+                'DeviceId': 1234, "Type": 1000},
+            {'Id': 234, 'TargetId': 235, 'ProfileState': 1, 'DeviceId': 1235, "Type": 1000}],
+            "report_list": [{'Id': 1234, 'PublicAddress': "XX.XX.XX.XX",
+                             'DeviceId': 1234, "Type": 1000}]},
+         "job_tracking": (True, "msg", {'LastRunStatus': {"Name": "Running"}}, True),
+            'message': "Changes found to be applied.",
+            'mparams': {"command": "deploy", "template_id": 123, "device_id": 1234},
+            "check_mode": True
+         },
+        {"json_data": {"value": [
+            {'Id': 123, 'TargetId': 123, 'ProfileState': 1,
+                'TemplateId': 1, 'DeviceId': 1234, "Type": 1000},
+            {'Id': 234, 'TargetId': 235, 'ProfileState': 1, 'TemplateId': 12, 'DeviceId': 1235, "Type": 1000}],
+            "report_list": [{'Id': 1234, 'PublicAddress': "XX.XX.XX.XX",
+                             'DeviceId': 1234, "Type": 1000}]},
+         "job_tracking": (True, "msg", {'LastRunStatus': {"Name": "Running"}}, True),
+            "get_device_ids": [123, 1234],
+            'message': "The device(s) '123' have been assigned the template(s) '1' respectively. Please unassign the profiles from the devices.",
+            'mparams': {"command": "deploy", "template_id": 123, "device_id": 1234}
+         },
+        {"json_data": {"value": [
+            {'Id': 123, 'TargetId': 123, 'ProfileState': 1,
+                'TemplateId': 123, 'DeviceId': 1234, "Type": 1000},
+            {'Id': 234, 'TargetId': 235, 'ProfileState': 1, 'TemplateId': 12, 'DeviceId': 1235, "Type": 1000}],
+            "report_list": [{'Id': 1234, 'PublicAddress': "XX.XX.XX.XX",
+                             'DeviceId': 1234, "Type": 1000}]},
+         "job_tracking": (True, "msg", {'LastRunStatus': {"Name": "Running"}}, True),
+            "get_device_ids": [123],
+            'message': "No changes found to be applied.",
+            'mparams': {"command": "deploy", "template_id": 123, "device_id": 1234}
+         },
+        {"json_data": {"value": [
+            {'Id': 123, 'TargetId': 123, 'ProfileState': 1,
+                'TemplateId': 123, 'DeviceId': 1234, "Type": 1000},
+            {'Id': 234, 'TargetId': 235, 'ProfileState': 1, 'TemplateId': 12, 'DeviceId': 1235, "Type": 1000}],
+            "report_list": [{'Id': 1234, 'PublicAddress': "XX.XX.XX.XX",
+                             'DeviceId': 1234, "Type": 1000}]},
+         "job_tracking": (True, "msg", {'LastRunStatus': {"Name": "Running"}}, True),
+            "get_device_ids": [123],
+            'message': "No changes found to be applied.",
+            'mparams': {"command": "delete", "template_id": 12, "device_id": 1234}
+         }
+    ])
+    def test_ome_template_success(self, params, ome_connection_mock_for_template, ome_response_mock,
+                                  ome_default_args, module_mock, mocker):
+        ome_response_mock.success = params.get("success", True)
+        ome_response_mock.json_data = params['json_data']
+        ome_connection_mock_for_template.get_all_report_details.return_value = params[
+            'json_data']
+        ome_default_args.update(params['mparams'])
+        mocks = ["job_tracking", "get_device_ids"]
+        for m in mocks:
+            if m in params:
+                mocker.patch(MODULE_PATH + m, return_value=params.get(m, {}))
+        result = self._run_module(
+            ome_default_args, check_mode=params.get('check_mode', False))
+        assert result['msg'] == params['message']
+
+    @pytest.mark.parametrize("params", [
+        {"json_data": {"value": [
+            {'Id': 123, 'TargetId': 123, 'ProfileState': 1,
+                'DeviceId': 1234, "Type": 1000},
+            {'Id': 234, 'TargetId': 235, 'ProfileState': 1, 'DeviceId': 1235, "Type": 1000}],
+            "report_list": [{'Id': 1234, 'PublicAddress': "XX.XX.XX.XX",
+                             'DeviceId': 1234, "Type": 1000}]},
+         "job_tracking": (True, "msg", {'LastRunStatus': {"Name": "Complete"}}, True),
+            'message': "Failed to deploy template.",
+            'mparams': {"command": "deploy", "template_id": 123, "device_id": 1234}
+         },
+        {"json_data": {"value": [
+            {'Id': 123, 'TargetId': 123, 'ProfileState': 1,
+                'DeviceId': 1234, "Type": 1000},
+            {'Id': 234, 'TargetId': 235, 'ProfileState': 1, 'DeviceId': 1235, "Type": 1000}],
+            "report_list": [{'Id': 1234, 'PublicAddress': "XX.XX.XX.XX",
+                             'DeviceId': 1234, "Type": 1000}]},
+         "job_tracking": (True, "msg", {'LastRunStatus': {"Name": "Complete"}}, True),
+         "get_device_ids": [],
+            'message': "There are no devices provided for deploy operation",
+            'mparams': {"command": "deploy", "template_id": 123, "device_id": 1234}
+         }
+    ])
+    def test_ome_template_fail_json(self, params, ome_connection_mock_for_template, ome_response_mock,
+                                    ome_default_args, module_mock, mocker):
+        ome_response_mock.success = params.get("success", True)
+        ome_response_mock.json_data = params['json_data']
+        ome_connection_mock_for_template.get_all_report_details.return_value = params[
+            'json_data']
+        ome_default_args.update(params['mparams'])
+        mocks = ["job_tracking", "get_device_ids"]
+        for m in mocks:
+            if m in params:
+                mocker.patch(MODULE_PATH + m, return_value=params.get(m, {}))
+        result = self._run_module(ome_default_args)
+        assert result['changed'] is False
+
     @pytest.mark.parametrize("exc_type",
                              [IOError, ValueError, TypeError, ConnectionError,
                               HTTPError, URLError, SSLError])
     def test_main_template_exception_case(self, exc_type, mocker, ome_default_args,
                                           ome_connection_mock_for_template, ome_response_mock):
-        ome_default_args.update({"command": "export", "template_name": "t1", 'attributes': {'Attributes': "myattr1"}})
+        ome_default_args.update(
+            {"command": "export", "template_name": "t1", 'attributes': {'Attributes': "myattr1"}})
         ome_response_mock.status_code = 400
         ome_response_mock.success = False
         json_str = to_text(json.dumps({"info": "error_details"}))
         if exc_type == URLError:
             mocker.patch(MODULE_PATH + 'password_no_log')
-            mocker.patch(MODULE_PATH + '_get_resource_parameters', side_effect=exc_type("url open error"))
+            mocker.patch(MODULE_PATH + '_get_resource_parameters',
+                         side_effect=exc_type("url open error"))
             result = self._run_module(ome_default_args)
             assert result["unreachable"] is True
         elif exc_type not in [HTTPError, SSLValidationError]:
-            mocker.patch(MODULE_PATH + '_get_resource_parameters', side_effect=exc_type("exception message"))
-            result = self._run_module_with_fail_json(ome_default_args)
+            mocker.patch(MODULE_PATH + '_get_resource_parameters',
+                         side_effect=exc_type("exception message"))
+            result = self._run_module(ome_default_args)
             assert result['failed'] is True
         else:
             mocker.patch(MODULE_PATH + '_get_resource_parameters',
-                         side_effect=exc_type('http://testhost.com', 400, 'http error message',
+                         side_effect=exc_type('https://testhost.com', 400, 'http error message',
                                               {"accept-type": "application/json"}, StringIO(json_str)))
-            result = self._run_module_with_fail_json(ome_default_args)
+            result = self._run_module(ome_default_args)
             assert result['failed'] is True
         assert 'msg' in result
