@@ -64,6 +64,9 @@ options:
         description: The rc codes list with which the conditions to fail will be overriden.
         type: list
         elements: int
+    revision_note:
+        description: The change note that can be specified when an object is created or updated.
+        type: str
     workspace_locking_adom:
         description: The adom to lock for FortiManager running in workspace mode, the value can be global and others including root.
         type: str
@@ -345,8 +348,8 @@ EXAMPLES = '''
     - name: Set various location based service
       fortinet.fortimanager.fmgr_wtpprofile_lbs:
         # bypass_validation: false
-        workspace_locking_adom: <value in [global, custom adom including root]>
-        workspace_locking_timeout: 300
+        # workspace_locking_adom: <global or your adom name>
+        # workspace_locking_timeout: 300
         # rc_succeeded: [0, -2, -3, ...]
         # rc_failed: [-2, -3, ...]
         adom: <your own value>
@@ -460,6 +463,7 @@ def main():
         'adom': {'required': True, 'type': 'str'},
         'wtp-profile': {'type': 'str', 'api_name': 'wtp_profile'},
         'wtp_profile': {'type': 'str'},
+        'revision_note': {'type': 'str'},
         'wtpprofile_lbs': {
             'type': 'dict',
             'v_range': [['6.0.0', '']],
